<?php

namespace Litecms\Location\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Litecms\Location\Forms\Country as CountryForm;
use Litecms\Location\Http\Requests\CountryResourceRequest;
use Litecms\Location\Http\Resources\CountryResource;
use Litecms\Location\Http\Resources\CountriesCollection;
use Litecms\Location\Models\Country;
use Litecms\Location\Scopes\CountryResourceScope;


/**
 * Resource controller class for country.
 */
class CountryResourceController extends BaseController
{

    /**
     * Initialize country resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = CountryForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('litecms.location.modules'), 'location', guard_url('location'));
    }

    /**
     * Display a list of country.
     *
     * @return Response
     */
    public function index(CountryResourceRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Country::pushScope(new RequestScope())
            ->pushScope(new CountryResourceScope())
            ->paginate($pageLimit);

        $data = new CountriesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('location::country.names'))
            ->view('location::country.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display country.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function show(CountryResourceRequest $request, Country $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CountryResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('location::country.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('location::country.show')
            ->output();
    }

    /**
     * Show the form for creating a new country.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CountryResourceRequest $request, Country $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CountryResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('location::country.name'))
            ->view('location::country.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new country.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CountryResourceRequest $request, Country $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new CountryResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('location::country.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('location/country/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/location/country'))
                ->redirect();
        }

    }

    /**
     * Show country for editing.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function edit(CountryResourceRequest $request, Country $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CountryResource($model);
        // return view('location::country.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('location::country.name'))
            ->view('location::country.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the country.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function update(CountryResourceRequest $request, Country $model)
    {
        try {
            $attributes = $request->all();
            $model->update($attributes);
            $data = new CountryResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('location::country.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('location/country/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('location/country/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the country.
     *
     * @param Model   $country
     *
     * @return Response
     */
    public function destroy(CountryResourceRequest $request, Country $model)
    {
        try {
            $model->delete();
            $data = new CountryResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('location::country.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('location/country/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('location/country/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
