<?php

namespace Litecms\Location\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Litecms\Location\Forms\Property as PropertyForm;
use Litecms\Location\Http\Requests\PropertyResourceRequest;
use Litecms\Location\Http\Resources\PropertyResource;
use Litecms\Location\Http\Resources\PropertiesCollection;
use Litecms\Location\Models\Property;
use Litecms\Location\Scopes\PropertyResourceScope;


/**
 * Resource controller class for property.
 */
class PropertyResourceController extends BaseController
{

    /**
     * Initialize property resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = PropertyForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('litecms.location.modules'), 'location', guard_url('location'));
    }

    /**
     * Display a list of property.
     *
     * @return Response
     */
    public function index(PropertyResourceRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Property::pushScope(new RequestScope())
            ->pushScope(new PropertyResourceScope())
            ->paginate($pageLimit);

        $data = new PropertiesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('location::property.names'))
            ->view('location::property.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display property.
     *
     * @param Request $request
     * @param Model   $property
     *
     * @return Response
     */
    public function show(PropertyResourceRequest $request, Property $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PropertyResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('location::property.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('location::property.show')
            ->output();
    }

    /**
     * Show the form for creating a new property.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PropertyResourceRequest $request, Property $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PropertyResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('location::property.name'))
            ->view('location::property.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new property.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PropertyResourceRequest $request, Property $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new PropertyResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('location::property.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('location/property/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/location/property'))
                ->redirect();
        }

    }

    /**
     * Show property for editing.
     *
     * @param Request $request
     * @param Model   $property
     *
     * @return Response
     */
    public function edit(PropertyResourceRequest $request, Property $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PropertyResource($model);
        // return view('location::property.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('location::property.name'))
            ->view('location::property.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the property.
     *
     * @param Request $request
     * @param Model   $property
     *
     * @return Response
     */
    public function update(PropertyResourceRequest $request, Property $model)
    {
        try {
            $attributes = $request->all();
            $model->update($attributes);
            $data = new PropertyResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('location::property.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('location/property/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('location/property/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the property.
     *
     * @param Model   $property
     *
     * @return Response
     */
    public function destroy(PropertyResourceRequest $request, Property $model)
    {
        try {
            $model->delete();
            $data = new PropertyResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('location::property.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('location/property/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('location/property/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
